"""Module for the action base class"""

class Base:
    """This class only provides default stubs for the availble methods"""

    def discover(self):
        """This is called before the main method to discover devices"""

    def while_on(self):
        """This is called in every loop, while a camera is on"""

    def switch(self, new_state):
        """This is called when the state switches between on/off (True/False)"""



"""Module to handle switching all Elgato lights"""

import leglight

from watch_webcam.actions.base import Base

class Light(Base):
    """Class to handle switching all Elgato lights"""

    def __init__(self, brightness=10, color=5500, discovery_timeout=10):
        self.brightness = brightness
        self.color = color
        self.discovery_timeout = discovery_timeout
        self.all_lights = []

    def discover(self):
        """Discover lights"""
        self.all_lights = leglight.discover(timeout=self.discovery_timeout)

        print(self.all_lights)

    def on(self):
        """Switch all lights on"""
        for light in self.all_lights:
            light.brightness(self.brightness)
            light.color(self.color)
            light.on()

    def off(self):
        """Switch all lights off"""
        for light in self.all_lights:
            light.off()

    def switch(self, new_state):
        """Switch lights depending on the state"""
        if new_state:
            self.on()
        else:
            self.off()



"""Module to pause media player"""

import re
import dbus
import dbus.service

from watch_webcam.actions.base import Base

class Media(Base):
    """Class to find and pause media player"""

    def find_player_service(self):
        """Find media service"""
        bus = dbus.SessionBus()
        for service in bus.list_names():
            if re.match('org.mpris.MediaPlayer2.', service):
                return dbus.SessionBus().get_object(service, '/org/mpris/MediaPlayer2')
        return None

    def pause_player(self):
        """Pause all media players"""

        player_service = self.find_player_service()
        if player_service:
            player = dbus.Interface(player_service, 'org.mpris.MediaPlayer2.Player')
            try:
                player.Pause()
            except dbus.DBusException as error:
                print("Can not find player? ", type(error).__name__, "–", error)

    def switch(self, new_state):
        """Overriding the base class to pause the player, whenever a camera is on"""
        if new_state:
            self.pause_player()



"""Module to control XScreenSaver"""

import subprocess

from watch_webcam.actions.base import Base

class XScreenSaver(Base):
    """Class to control XScreenSaver"""

    def deactivate(self):
        """Deactivate XScreenSaver - pretending that the user is active"""
        return subprocess.run(
            ['xscreensaver-command', '-deactivate'],
            stdout=subprocess.DEVNULL,
            check=False
        )

    def while_on(self):
        """Overriding base method, which is called while camera is on"""
        self.deactivate()



#!/bin/env python3

"""Main entry point for the watch-webcam"""

import argparse
import time
import yaml

from watch_webcam.video import Video

from watch_webcam.actions.xscreensaver import XScreenSaver
from watch_webcam.actions.media import Media
from watch_webcam.actions.light import Light

ACTIONS=[XScreenSaver, Media, Light]

def main():
    """Main entry point for the watch-webcam"""

    parser = argparse.ArgumentParser(
        prog="watch-webcam",
        description="Detects webcam use and acts on it"
    )
    parser.add_argument("-c", "--config", default="watch-webcam.yml")
    args = parser.parse_args()

    with open(args.config, "r", encoding="utf-8") as file:
        config = yaml.safe_load(file)

    video = Video(**config["video"])

    actions = []
    for action in ACTIONS:
        params = config[action.__name__.lower()]
        if params["enabled"]:
            del params["enabled"]
            actions.append(action(**params))

    for action in actions:
        action.discover()

    previous_state = None
    while True:
        new_state = video.is_on()
        if new_state:
            for action in actions:
                action.while_on()

        if new_state != previous_state:
            for action in actions:
                action.switch(new_state)

            previous_state = new_state
        else:
            time.sleep(1)


if __name__ == '__main__':
    main()



"""This module checks video devices for access by video call software"""

import subprocess

class Video:
    """This class handles checking the applications accessing video devices"""

    DEFAULT_DEVICES = ["/dev/video0", "/dev/video1", "/dev/video2"]
    DEFAULT_APPLICATIONS = [
            "cheese",
            "MainThread",
            "firefox",
            "zoom",
            "GeckoMain",
            "teams",
            "chrome",
            "slack",
        ]

    def __init__(self, devices=None, applications=None):
        if devices is None:
            devices = Video.DEFAULT_DEVICES
        if applications is None:
            applications = Video.DEFAULT_APPLICATIONS
        self.devices = devices
        self.applications = applications

    def fuser(self, filename):
        """Returns fuser string for device"""
        return subprocess.run(
            ['fuser', '-v', filename],
            check=False,
            capture_output=True).stderr.decode("utf-8")
    def video_state(self, filename):
        """Returns if the video device is currently accessed by one of the applications"""

        output = self.fuser(filename)
        return any(output.find(program)>=0 for program in self.applications)

    def is_on(self):
        """Checks if any of the devices are currently accessed by one of the applications"""
        return any(self.video_state(filename) for filename in self.devices)
